//Ensures that this file isn't included twice within a translation unit.
#ifndef JCAM_DLL
#define JCAM_DLL

#include <stddef.h>

#if defined WIN32
#include <atlsafe.h>

	#if (defined JCAM_DYNAMIC_LIB && ! defined JCAM_STATIC_LIB)	
		// The following ifdef block is the standard way of creating macros which make exporting
		// from a DLL simpler. All files within this DLL are compiled with the JCAM_DLL_EXPORTS
		// symbol defined on the command line. This symbol should not be defined on any project
		// that uses this DLL. This way any other project whose source files include this file see
		// JCAM_DLL_API functions as being imported from a DLL, whereas this DLL sees symbols
		// defined with this macro as being exported.

		#if defined JCAM_DLL_EXPORTS
			#define JCAM_DLL_API extern "C" __declspec(dllexport)
		#else
			#define JCAM_DLL_API __declspec(dllimport)
		#endif

		#define STDCALL __stdcall    // Declare our calling convention.
	#elif (! defined JCAM_DYNAMIC_LIB && defined JCAM_STATIC_LIB)
		#define JCAM_DLL_API
		#define STDCALL __stdcall
	#else
		#error "You need to define either JCAM_DYNAMIC_LIB if you will distribute jcam_dll.dll with your software, or JCAM_STATIC_LIB if you're statically linking with jcam_dll.lib."
	#endif

#else //WIN32

	#define JCAM_DLL_API
	#define STDCALL
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <netinet/tcp.h>
#endif //WIN32

#ifdef __cplusplus
	//Tells the compiler to use C symbol naming conventions
	extern "C" {

	//Put things in a separate namespace so that there's no chance of collisions
	namespace joescan {
#endif //__cplusplus

#ifndef SWIG
#define CHECK_ELEMENT_SIZE(element, size) \
	extern char element ## size[static_cast<int>((sizeof(element) == (size)))]

CHECK_ELEMENT_SIZE(char, 1);
CHECK_ELEMENT_SIZE(short, 2);
CHECK_ELEMENT_SIZE(int, 4);
//CHECK_ELEMENT_SIZE(size_t, 4);
#endif

#ifndef INT8
/// Signed 8 bit integer.
typedef signed char         INT8;
/// Signed 16 bit integer.
typedef signed short        INT16;
/// Signed 32 bit integer.
typedef signed int          INT32;
/// Unsigned 8 bit integer.
typedef unsigned char       UINT8;
/// Unsigned 16 bit integer.
typedef unsigned short      UINT16;
/// Unsigned 32 bit integer.
typedef unsigned int        UINT32;
#endif

/**
 * \brief Constants that avoid the traditional problems of \c \#define constants.
 */
enum jsConstants
{
	/** \brief An operation was unable to complete. */
	OPERATION_FAILURE   = 21,
	/** \brief The connection to the scanner is invalid. */
	SCANNER_FAILURE     = -1,
	/** \brief A parameter to a function was \c NULL or out of range. */
	INVALID_PARAMETER   = -2,
	/** \brief A scanner in Synchronized Scanning Mode has no available profiles. */
	PROFILE_UNAVAILABLE = -3,
	/** \brief A connection to a scanner timed out. */
	SOCKET_TIMEOUT      = -4,

	/** \brief If this bit is set in a \c #jsProfile's flags field, then Synchronized Scanning Mode is running the scanner too fast. */
	SYNC_MODE_OVERRUN = 0x00000001,

	/** \brief CCD sensor dimensions. */
	MAX_HORIZONTAL = 324,
	/** \brief CCD sensor dimensions. */
	MAX_VERTICAL   = 243,

	/** \brief Static IP Address for a scanner. */
	IPS_STATIC    = 0,
	/** \brief IP Address is determined by the cable ID. */
	IPS_BY_ID     = 1,
	/** \brief IP Address is assigned with DHCP. */
	IPS_BY_DHCP   = 2,
	/** \brief The scanner is fine. */
	STATUS_NORMAL = 0,
	/** \brief The scanner is scanning. */
	STATUS_ACTIVE = 1,
	/** \brief The number of bytes in a \c #jsResponsePacket's \c options field. */
	OPTIONS_SIZE  = 512,
	/** \brief The maximum length of the \c date field in a \c jsOldCalibrationValue. */
	DATE_LENGTH = 32,
};

/**
 * \brief A type for specifying which laser to use.
 *
 * If an invalid laser is specified, the connection will be closed.
 */
typedef enum jsLaserIndexTag
{
	/** \brief Furthest from the camera. */
	LASER0 = 0,
	/** \brief Closer to the camera than LASER0. */
	LASER1 = 1,
	/** \brief Closer to the camera than LASER1. */
	LASER2 = 2,
	/** \brief Closer to the camera than LASER2. */
	LASER3 = 3,
	/** \brief Closer to the camera than LASER3. */
	LASER4 = 4,
} jsLaserIndex;

/**
 * \brief An opaque handle for connections to scanners.
 *
 * \c #JCONNECTION's are full-duplex request/reply connections.
 * Functions have been provided that handle all the request/reply semantics.
 */
typedef void* JCONNECTION;

#pragma pack(1) //Tells compiler to use one-byte boundaries for packing structs.

/** \brief Represents a coordinate and its associated pixel brightness. */
typedef struct tagProfileDataPoint
{
	/** \brief The x coordinate of the data point in 1000ths of an inch. */
	INT32 x;
	/** \brief The y coordinate of the data point in 1000ths of an inch. */
	INT32 y;
	/**
	 * \brief The intensity of the pixel where the laser was detected.
	 *
	 * The value is the sum of 7 pixels, each with 10 bits of resolution.
	 * Consequently, the maximum value is 7 * 2<SUP>10</SUP> = 7168.
	 * If you want to display this value in a traditional 255 level greyscale, you will have to map the brightness value to a 255 level greyscale on your own.
	 */
	INT32 brightness;
} ProfileDataPoint;

/**
 * \brief Represents a subpixel point from a scanner and its brightness.
 */
typedef struct tagScanDataPoint
{
	/**
	 * \brief Which subpixel the laser was centered at.
	 *
	 * Range is from 0 to 32,400 because there are 100 subpixels per pixel and 324 pixels.
	 */
	INT32 data;
	/**
	 * \brief A simple representation of the intensity of the pixel.
	 *
	 * 0 means invalid.<br>
	 * 1 means normal.<br>
	 * 2 means bright.<br>
	 * 3 means very bright.
	 */
	INT32 status;
	/**
	 * \brief The actual intensity of the pixel where the laser was detected.
	 *
	 * The value is the sum of 7 pixels, each with 10 bits of resolution.
	 * Consequently, the maximum value is 7 * 2<SUP>10</SUP> = 7168.
	 * If you want to display this value in a traditional 255 level greyscale.
	 * you will have to map the brightness value to a 255 level greyscale on your own.
	 */
	INT32 brightness;
} ScanDataPoint;

/**
 * \brief Profile data from the scanner.
 */
typedef struct tagProfile
{
	/**
	 * \brief Sequence number from Synchronized Scanning Mode.
	 *
	 * Sequential \c #jsProfile's during Synchronized Scanning Mode should have sequential \c sequenceNumber's.
	 * The counter is reset to 0 every time Synchronized Scanning Mode is restarted.
	 */
	INT32 sequenceNumber;
	/** \brief The encoder value when when the scanner created the profile. */
	INT32 location;
	/** \brief The encoder value when the scanner sent the profile. */
	INT32 sendLocation;
	/** \brief The number of 10 microsecond units the laser was on. */
	INT32 laserOnTime;
	/** \brief The time since the scanner was turned on.
	 *
	 * It is not the time of day, nor the current date.
	 */
	INT32 timeInHead;
	/**
	* \brief The status of all cable inputs to the scanner.
	*
	* This 32 bit integer holds status of the encoder inputs and Start Scan line at the time
	*  the scan was taken.
	* The value for the differential encoder inputs (0x20 and 0x40) should not be used by client code.
	* The value of the Start Scan line is in bit 8 (0x80) and will be set to 1 for profiles
	* taken while the line was pulled low, and 0 otherwise.
	*/
	INT32 inputs;
	/**
	 * \brief Information from the scanner.
	 *
	 * If bit 0 is set, then Synchronized Scanning Mode scanning is trying to run faster than the scanner is capable and
	 * some data will be lost on one end of the laser line.<BR/>
	 * If bit 1 is set, this indicates that an internal error has occured in the scanner. Contact a JoeScan support
	 * engineer if you encounter this flag.<BR/>
	 * The remaining bits are reserved and values may change in future DLLs.  So you should mask them to be ensure future firmware
	 * revision don't break your software.
	 */
	INT32 flags;
	/** \brief Currently unused. */
	INT32 laserIndex;
	/** \brief Which laser this profile is from. */
	INT32 reserved2;
	/** \brief The number of valid coordinate points in this profile. */
	INT32 numberPoints;
	/**
	 * \brief Coordinates in 1000ths of an inch; i.e. 2000 is 2 inches. 
	 * If units are set to metric units are micrometers, and 2000 in 2mm.
	 *
	 * \c #MAX_VERTICAL is defined in \c #jsConstants.
	 */
	ProfileDataPoint data[MAX_VERTICAL];
} jsProfile;

/**
 * \brief Image data from the scanner.
 */
typedef struct tagImage
{
	/** \brief How many 10 microsecond units the camera was exposed. */
	INT32 exposureTime;
	/** \brief Currently unused. */
	INT32 reserved1;
	/** \brief Currently unused. */
	INT32 reserved2;
	/** \brief The number of horizontal pixels. */
	INT32 numberHorizontal;
	/** \brief The number of vertical pixels. */
	INT32 numberVertical;
	/**
	 * \brief The image pixels.
	 *
	 * \c #MAX_VERTICAL and \c #MAX_HORIZONTAL are defined in \c #jsConstants.
	 */
	UINT8 image[MAX_HORIZONTAL * MAX_VERTICAL];
} jsImage;

/**
 * \brief Scan data from the scanner.
 */
typedef struct tagScan
{
	/** \brief The encoder value when the scan was taken. */
	INT32 location;
	/** \brief The encoder value when the scan was sent. */
	INT32 sendLocation;
	/** \brief The number of 10 microseconds units the laser was on. */
	INT32 laserOnTime;
	/**
	 * \brief The time since the scanner was turned on.
	 *
	 * It is not the time of day, nor the current date.
	 */
	INT32 timeInHead;
	/**
	 * \brief The cable input to the scanner.
	 *
	 * This includes the Cable ID, the encoder signals, and the Start Scan signal.
	 */
	INT32 inputs;
	/**
	 * \brief Information from the scanner.
	 *
	 * If bit 0 is set, then Synchronized Scanning Mode scanning is trying to run faster than the scanner is capable.
	 * The rest of the bits are unused and should be 0.
	 */
	INT32 flags;
	/** \brief Currently unused. */
	INT32 reserved1;
	/** \brief Currently unused. */
	INT32 reserved2;
	/** \brief The number of valid pixel points in this profile. */
	INT32 numberPoints;
	/**
	 * \brief Pixel data points.
	 *
	 * \c #MAX_VERTICAL is defined in \c #jsConstants.
	 */
	ScanDataPoint data[MAX_VERTICAL];
} jsScan;

/**
 * \brief Responses from scanners on the network.
 */
typedef struct tagResponsePacket
{
	/** \brief The Ethernet hardware MAC address of the scanner. */
	UINT8  macAddress[6];
	/**
	 * \brief The current addressing technique the scanner is using.
	 *
	 * \c #IPS_STATIC Static IP Address for the scanner.<br>
	 * \c #IPS_BY_ID IP Address is determined by the cable ID.<br>
	 * \c #IPS_BY_DHCP IP Address is assigned with DHCP.
	 */
	UINT8  currentIpSetup;
	/** \brief The IP Address of the responding scanner, in <B>network byte order</B>. */
	UINT32 ipAddress;
	/** \brief The serial number of the responding scanner. */
	UINT32 serialNumber;
	/** \brief The cable ID of the responding scanner. */
	UINT8  cableId;
	/** \brief The status of the responding scanner. */
	UINT8  status;
	/** \brief The firmware revision number of the responding scanner. */
	UINT32 build;
	/**
	 * \brief Internal use.  Don't change the contents.
	 *
	 * \c #OPTIONS_SIZE is defined in \c #jsConstants.
	 */
	INT8   options[OPTIONS_SIZE];
} jsResponsePacket;

/**
 * \brief Old position calibration parameters sent to the scanner.
 */
typedef struct tagOldCalibrationValue
{
	/** \brief The X offset of the scanner's position. */
	double x;
	/** \brief The Y offset of the scanner's position. */
	double y;
	/** \brief The scanner's rotation. */
	double roll;
	/** \brief The date the calibration was sent.  This can be an empty string. */
	char date[DATE_LENGTH];
} jsOldCalibrationValue;

#pragma pack() //Use compiler default packing after this.

/**
 * \ingroup miscellaneousFunctions
 * \brief Returns the major revision number of the DLL.
 */
JCAM_DLL_API int STDCALL jsGetJcamDllMajorVersionNumber();
/**
 * \ingroup miscellaneousFunctions
 * \brief Returns the minor revision number of the DLL.
 */
JCAM_DLL_API int STDCALL jsGetJcamDllMinorVersionNumber();
/**
 * \ingroup managingConnections
 * \brief Opens a connection to the specified host.
 *
 * \b All successful calls to this function \b must be closed with a call to \c jsCloseConnection().
 *
 * \see \c #jsCloseConnection()
 *
 * \param host Can be in either xxx.xxx.xxx.xxx format or can be a DNS name for lookup.
 *
 * \return The \c #JCONNECTION on success.
 * \return \c NULL if a failure occurred.
 */
JCAM_DLL_API JCONNECTION STDCALL jsOpenConnection(char const * const host);
/**
 * \ingroup managingConnections
 * \brief Opens a connection to the specified host.
 *
 * \b All successful calls to this function \b must be closed with a call to \c jsCloseConnection().
 *
 * \see \c #jsCloseConnection()
 *
 * \param host <b>Must be in network byte order.</b> The best way to get this is by using the \ref scannerDiscovery "Scanner Discovery" functions.
 *
 * \return The \c #JCONNECTION on success.
 * \return \c NULL if a failure occurred.
 */
JCAM_DLL_API JCONNECTION STDCALL jsOpenConnectionInt(int const host);
/**
 * \ingroup managingConnections
 * \brief Opens a connection to the specified scanner using Base IP + Cable ID addressing.
 *
 * \b All successful calls to this function \b must be closed with a call to \c jsCloseConnection().
 *
 * \see \c #jsCloseConnection()
 *
 * \param host Must be in xxx.xxx.xxx.xxx format.
 * \param cableID The Cable ID of the scanner to connect to.
 *
 * \return The \c #JCONNECTION on success.
 * \return \c NULL if a failure occurred.
 */
JCAM_DLL_API JCONNECTION STDCALL jsOpenConnectionBase(char const * const host, UINT32 const cableID);
/**
 * \ingroup managingConnections
 * \brief Closes a \c #JCONNECTION.
 *
 * It is safe to pass in \c NULL.
 *
 * If a function returns \c #SCANNER_FAILURE, you must close the
 * connection with this function.  \c #JCONNECTION's are allocated data
 * structures that must be properly deallocated, otherwise there will be
 * memory leaks in your program.
 *
 * \see \c #jsOpenConnection()
 * \see \c #jsOpenConnectionInt()
 * \see \c #jsOpenConnectionBase()
 *
 * \param jc The connection to be closed.
 *
 * \return 0 on success.
 * \return -1 on all failures.
 */
JCAM_DLL_API int STDCALL jsCloseConnection(JCONNECTION const jc);
/**
 * \ingroup laserSpecific
 * \brief Sends a request for a profile from the specified laser.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * Nothing is read from the connection during this call.
 * Increments the number of outstanding requests for this connection by one.
 *
 * \param jc The connection to send the request to.
 * \param laserIndex  Which laser to use for the scan.
 *
 * \see \c #jsReadProfileN()
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsCleanUpOutstandingRequests()
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsSendProfileRequestN(JCONNECTION const jc, jsLaserIndex const laserIndex);
/**
 * \ingroup laserSpecific
 * \brief Reads a profile from the connection.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * A call to \c jsSendProfileRequestN() must have been made to the specified
 * \c #JCONNECTION before this function is called.  Otherwise the call will
 * block and eventually time out waiting for a profile that isn't coming.
 *
 * Decrements the number of outstanding requests for this connection by one.
 *
 * \see \c #jsSendProfileRequestN()
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsCleanUpOutstandingRequests()
 *
 * \param jc The connection to read the profile from.
 * \param profile Where the profile is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c profile is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #SOCKET_TIMEOUT if the read took longer than 5 seconds.
 */
JCAM_DLL_API int STDCALL jsReadProfileN(JCONNECTION const jc, jsProfile * const profile);
/**
 * \ingroup laserSpecific
 * \brief Requests and reads a profile from the specified laser.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \param jc The connection to read a profile from.
 * \param laserIndex Which laser to use for the scan.
 * \param profile Where the profile is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c profile is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #SOCKET_TIMEOUT if the read took longer than 5 seconds.
 */
JCAM_DLL_API int STDCALL jsGetProfileN(JCONNECTION const jc, jsLaserIndex const laserIndex, jsProfile * const profile);
/**
 * \ingroup laserSpecific
 * \brief Requests and reads a profile from each laser the scanner has.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * If \c cProfiles indicates \c profiles doesn't have enough space, then extra profiles are discarded.
 * Really, \c profiles should point to an array with enough space for a profile from every laser.
 *
 * \param jc The connection to read the profiles from.
 * \param profiles Pointer to an array of \c #jsProfile's.
 * \param cProfiles The number of \c #jsProfile's that \c profiles points to.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c profile is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #SOCKET_TIMEOUT if the read took longer than 5 seconds.
 */
JCAM_DLL_API int STDCALL jsGetProfileFromAllLasers(JCONNECTION const jc, jsProfile * const profiles, size_t const cProfiles);
/**
 * \ingroup scanAndImage
 * \brief Requests and reads an image.
 *
 * \param jc The connection to read the image from.
 * \param image Where the image is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c image is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsGetImage(JCONNECTION const jc, jsImage * const image);
/**
 * \ingroup laserSpecific
 * \brief Requests and reads an image with the specified laser on.
 *
 * \param jc The connection to read the image from.
 * \param laserIndex Which laser to use for the image.
 * \param image Where the image is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c image is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 */
JCAM_DLL_API int STDCALL jsGetImageN(JCONNECTION const jc, jsLaserIndex const laserIndex, jsImage * const image);
/**
 * \ingroup scanAndImage
 * \brief Requests and reads a scan.
 *
 * \param jc The connection to read the scan from.
 * \param scan Where the scan is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c scan is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.
 */

JCAM_DLL_API int STDCALL jsGetScan(JCONNECTION const jc, jsScan * const scan);
/**
 * \ingroup laserSpecific
 * \brief Requests and reads a scan with the specified laser on.
 *
 * \param jc The connection to read the scan from.
 * \param laserIndex Which laser to use for the scan.
 * \param scan Where the scan is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c scan is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.
 */
JCAM_DLL_API int STDCALL jsGetScanN(JCONNECTION const jc, jsLaserIndex const laserIndex, jsScan * const scan);
/**
 * \ingroup scanAndImage
 * \brief Requests and reads an image and scan.
 *
 * \param jc The connection to read the image and scan from.
 * \param image Where the image is stored.
 * \param scan Where the scan is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c scan is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsGetImageScan(JCONNECTION const jc, jsImage * const image, jsScan * const scan);
/**
 * \ingroup laserSpecific
 * \brief Requests and reads an image and scan with the specified laser on.
 *
 * \param jc The connection to read the image and scan from.
 * \param laserIndex Which laser to use for the image and scan.
 * \param image Where the image is stored.
 * \param scan Where the scan is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c scan is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 */
JCAM_DLL_API int STDCALL jsGetImageScanN(JCONNECTION const jc, jsLaserIndex const laserIndex, jsImage * const image, jsScan * const scan);
/**
* \ingroup laserSpecific
* \brief Requests and reads a scan with the specified laser and camera timings. This function requires firmware revision 13817 and higher.
*
* \param jc The connection to read the scan from.
* \param cameraExposure The time in microseconds to expose the camera.
* \param laserExposure The time in microseconds to expose the laser.
* \param laserIndex Which laser to use for the scan.
* \param scan Where the scan is stored.
*
* \return 0 on success.
* \return \c #INVALID_PARAMETER if \c jc or \c scan is \c NULL.
* \return \c #SCANNER_FAILURE on all connection failures.
*
* May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
*/
JCAM_DLL_API int STDCALL jsGetDiagnosticScan(JCONNECTION const jc, int const cameraExposure, int const laserExposure, const jsLaserIndex laserIndex, jsScan * const scan);
/**
* \ingroup laserSpecific
* \brief Requests and reads an image and scan with the specified laser and camera timings. This function requires firmware revision 13817 and higher.
*
* \param jc The connection to read the image and scan from.
* \param cameraExposure The time in microseconds to expose the camera.
* \param laserExposure The time in microseconds to expose the laser.
* \param laserIndex Which laser to use for the image and scan.
* \param image Where the image is stored.
* \param scan Where the scan is stored.
*
* \return 0 on success.
* \return \c #INVALID_PARAMETER if \c jc or \c scan is \c NULL.
* \return \c #SCANNER_FAILURE on all connection failures.
*
* May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
*/
JCAM_DLL_API int STDCALL jsGetDiagnosticImageScan(JCONNECTION const jc, int const cameraExposure, int const laserExposure, const jsLaserIndex laserIndex, jsImage * const image, jsScan * const scan);
/**
 * \ingroup synchronizedScanning
 * \brief Causes the scanner to enter Start Scan Triggered Synchronized Scanning Mode.
 *
 * While in Pulse Triggered Synchronized Scanning Mode, the scanners will trigger on the appropriate edge of the Start Scan signal, depending on the presence of \c StartScanTriggerOnHigh.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsExitSyncMode() 
 * \see \c #jsStartPulseMaster()
 * \see \c #jsStopPulses()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsEnterPulseSyncMode(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Causes the scanner to enter Encoder Synchronized Scanning Mode.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsExitSyncMode()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsEnterEncoderSyncMode(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Causes the scanner to enter Time Synchronized Scanning Mode.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsExitSyncMode()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsEnterTimeSyncMode(JCONNECTION const jc);

/**
 * \ingroup synchronizedScanning
 * \brief Causes the scanner to output a pulse train on the Start Scan I/O.  The period and number of pulses are configurable
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsStopPulses()
 *
 * \param jc The connection to the scanner.
 * \param pulseInterval The period in microseconds of the pulse train. Between 1,000 and 5,000,000.
 * \param pulseCount The number of pulses to send before stopping.  If zero pulses will be continuously sent.  Max value is 511.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL, or other parameters are out of range.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsStartPulseMaster(JCONNECTION const jc, int pulseInterval, int pulseCount);
/**
 * \ingroup synchronizedScanning
 * \brief Causes the scanner to stop generating a pulse train on the Start Scan I/O.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsStartPulseMaster()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsStopPulses(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Set the alternating exposure parameters.  Only works in sync mode.  Settings are lost when exiting sync mode and must be resent after re-entering syncmode.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \param jc The connection to the scanner.
 * \param enable If true enable the alternating exposure mode, if false return to normal exposure mode.
 * \param exposure1 One of the two exposure times, must be between 10 and 650000.
 * \param exposure2 One of the two exposure times, must be between 10 and 650000
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL, or other parameters are out of range.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsSetAlternatingExposure(JCONNECTION const jc, bool const enable, int const exposure1, int const exposure2);
/**
 * \ingroup synchronizedScanning
 * \brief Causes the scanner to halt Time or Encoder Synchronized Scanning Mode.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * Any scans queued up in the scanner can still be read using \c jsGetProfile().
 * The scanner is still in Synchronized Scanning Mode after calling this function.
 * A call to \c jsExitSyncMode() must follow in order to exit Synchronized Scanning Mode.
 *
 * \see \c #jsExitSyncMode()
 * \see \c #jsEnterEncoderSyncMode()
 * \see \c #jsEnterTimeSyncMode()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsHaltSyncMode(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Causes the scanner to exit Time or Encoder Synchronized Scanning Mode.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * Any profiles queued in the scanner will be lost.
 * This functionalso calls \c #jsCleanUpOutstandingRequests() in order to fully reset scanning.
 *
 * \see \c #jsEnterEncoderSyncMode()
 * \see \c #jsEnterTimeSyncMode()
 * \see \c #jsHaltSyncMode()
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsCleanUpOutstandingRequests()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsExitSyncMode(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Sends a request for up to \c nProfiles profiles to the scanner while in Synchronized Scanning Mode.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * No profiles are read from the connection during this call.
 * In order to read the profiles, a call to \c jsReadMultipleProfiles() must follow.
 *
 * If the parameters attempt to run the scanner faster than it can physically scan,
 * then bit 0 in the flags field of the \c #jsProfile will be set.
 *
 * Increments the number of outstanding requests for this connection by one.
 *
 * \see \c #jsReadMultipleProfiles()
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsCleanUpOutstandingRequests()
 *
 * \param jc The connection to the scanner.
 * \param nProfiles The number of profiles to request from the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsSendMultipleProfileRequest(JCONNECTION const jc, UINT32 const nProfiles);
/**
 * \ingroup synchronizedScanning
 * \brief Reads up to \c nProfiles profiles from \c jc after a
 * \c jsSendMultipleProfileRequest() call in Synchronized Scanning Mode.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.
 *
 * \c nProfiles must be equal to the number of profiles
 * requested in the corresponding call to \c jsSendMultipleProfileRequest(),
 * and \c profiles must point to an array of \c nProfiles \c #jsProfile structs.
 *
 * If the parameters attempt to run the scanner faster than it can physically scan,
 * then bit 0 in the flags field of the \c #jsProfile's will be set.
 *
 * Decrements the number of outstanding requests for this connection by one.
 *
 * \see \c #jsSendMultipleProfileRequest()
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsCleanUpOutstandingRequests()
 *
 * \param jc The connection to the scanner.
 * \param profiles Pointer to an array of \c #jsProfile's.
 * \param nProfiles The number of \c #jsProfile's that \c profiles points to.
 *
 * \return The number of valid profiles on success.
 * \return 0 if no profiles are available.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c profiles is \c NULL.
 * \return \c #SOCKET_TIMEOUT if the read took longer than 5 seconds.
 */
JCAM_DLL_API int STDCALL jsReadMultipleProfiles(JCONNECTION const jc, jsProfile * const profiles, UINT32 nProfiles);
/**
 * \ingroup synchronizedScanning
 * \brief Requests and reads up to n profiles from the scanner while in either Synchronized Scanning Mode.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * If the parameters attempt to run the scanner faster than it can physically scan,
 * then bit 0 in the flags field of the \c #jsProfile's will be set.
 *
 * \param jc The connection to the scanner.
 * \param profiles Pointer to an array of \c #jsProfile's.
 * \param cProfiles The number of \c #jsProfile's that \c profiles points to.  Also how many profiles are requested from the scanner.
 * \param timeoutMilliseconds The maximum number of milliseconds that a read is allowed to take.
 *
 * \return The number of valid profiles on success.
 * \return 0 if no profiles are available.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c profiles is \c NULL.
 * \return \c #SOCKET_TIMEOUT if the read took longer than the specified timeout period.
 */
JCAM_DLL_API int STDCALL jsReadMultipleProfilesTimeout(JCONNECTION const jc, jsProfile * const profiles, UINT32 cProfiles, long timeoutMilliseconds);
/**
 * \ingroup synchronizedScanning
 * \brief Requests and reads up to n profiles from the scanner while in either Synchronized Scanning Mode.
 *
 * May \b only be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * If the parameters attempt to run the scanner faster than it can physically scan,
 * then bit 0 in the flags field of the \c #jsProfile's will be set.
 *
 * \param jc The connection to the scanner.
 * \param profiles Pointer to an array of \c #jsProfile's.
 * \param cProfiles The number of \c #jsProfile's that \c profiles points to.  Also how many profiles are requested from the scanner.
 *
 * \return The number of valid profiles on success.
 * \return 0 if no profiles are available.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c profiles is \c NULL.
 * \return \c #SOCKET_TIMEOUT if the read took longer than 5 seconds.
 */
JCAM_DLL_API int STDCALL jsGetMultipleProfiles(JCONNECTION const jc, jsProfile * const profiles, UINT32 const cProfiles);
/**
 * \ingroup synchronizedScanning
 * \brief Sends a request for a profile. 
 *
 * May be called while the scanner is or is not in Synchronized Scanning Mode.<br>
 *
 * Nothing is read from the connection during this call.
 * Increments the number of outstanding requests for this connection by one.
 *
 * \param jc The connection to send the request to.
 *
 * \see \c #jsReadProfileN()
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsCleanUpOutstandingRequests()
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsSendProfileRequest(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Reads a profile from the connection.
 *
 * May be called while the scanner is or is not in Synchronized Scanning Mode.<br>
 *
 * A call to \c jsSendProfileRequest() must have been made to the specified
 * \c #JCONNECTION before this function is called.  Otherwise the call will
 * block and eventually time out waiting for a profile that isn't coming.
 *
 * Decrements the number of outstanding requests for this connection by one.
 *
 * \see \c #jsSendProfileRequestN()
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsCleanUpOutstandingRequests()
 *
 * \param jc The connection to read the profile from.
 * \param profile Where the profile is stored.
 *
 * \return 0 or \c #PROFILE_UNAVAILBLE on success.
 * \return \c #PROFILE_UNAVAILBLE if in sync mode and the profile queue is empty
 * \return \c #INVALID_PARAMETER if \c jc or \c profile is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #SOCKET_TIMEOUT if the read took longer than 5 seconds.
 */
JCAM_DLL_API int STDCALL jsReadProfile(JCONNECTION const jc, jsProfile * const profile);
/**
 * \ingroup synchronizedScanning
 * \brief Requests and reads a profile from the default laser.
 *
 * May be called while the scanner is or is not in Synchronized Scanning Mode.<br>
 *
 * \param jc The connection to read a profile from.
 * \param profile Where the profile is stored.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c profile is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #PROFILE_UNAVAILABLE if the scanner is in Synchronized Scanning Mode and there is no profile available.
 * \return \c #SOCKET_TIMEOUT if the read took longer than 5 seconds.
 */
JCAM_DLL_API int STDCALL jsGetProfile(JCONNECTION const jc, jsProfile * const profile);
/**
 * \ingroup synchronizedScanning
 * \brief Returns the number of unread requests from either \c #jsSendProfileRequestN() or \c #jsSendMultipleProfileRequest().
 *
 * Returns the number of unread requests generated by either \c #jsSendProfileRequestN() or \c #jsSendMultipleProfileRequest().
 * This function does not differentiate between requests generated by \c #jsSendProfileRequestN() or \c #jsSendMultipleProfileRequest().<br>
 *
 * The number of profiles generated by \c #jsSendMultipleProfileRequest() may be higher than the number of outstanding requests since more than one profile could be returned by one request.
 *
 * Any further calls to \c #jsReadProfileN() or \c #jsReadMultipleProfiles() will block unless a new call to \c #jsSendProfileRequestN() or \c #jsSendMultipleProfileRequest() is made.
 *
 * \see \c #jsCleanUpOutstandingRequests()
 * \see \c #jsSendProfileRequestN()
 * \see \c #jsReadProfileN()
 * \see \c #jsSendMultipleProfileRequest()
 * \see \c #jsReadMultipleProfiles()
 *
 * \param jc The connection to ask how many unread responses there are.
 *
 * \return The number of unread responses from either \c #jsSendProfileRequestN() or \c #jsSendMultipleProfileRequest().
 */
JCAM_DLL_API size_t STDCALL jsGetNumberOfOutstandingRequests(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Reads and discards all the profiles in the TCP/IP stack generated by \c #jsSendProfileRequestN() or \c #jsSendMultipleProfileRequest().
 *
 * Reads any outstanding responses to \c #jsSendProfileRequestN() or \c #jsSendMultipleProfileRequest() calls.
 *
 * \see \c #jsGetNumberOfOutstandingRequests()
 * \see \c #jsSendProfileRequestN()
 * \see \c #jsReadProfileN()
 * \see \c #jsSendMultipleProfileRequest()
 * \see \c #jsReadMultipleProfiles()
 *
 * \param jc The connection to discard unread profiles from.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsCleanUpOutstandingRequests(JCONNECTION const jc);
/**
 * \ingroup synchronizedScanning
 * \brief Sets the scanner's internal encoder value.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \param jc The connection to the scanner.
 * \param newEncoderValue The new encoder value for the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsSetEncoderValue(JCONNECTION const jc, UINT16 newEncoderValue);
/**
 * \ingroup synchronizedScanning
 * \brief Sets the scanner's internal encoder value.
 *
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \param jc The connection to the scanner.
 * \param newEncoderValue The new encoder value for the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsSetEncoderValue32(JCONNECTION const jc, UINT32 newEncoderValue);
/**
 * \ingroup parameters
 * \brief Starts sending a parameter file to the scanhead.  Must be followed with a called to \c jsEndSendParameterFileToScanner().
 * 
 * When connecting to large group of scanners, it is much faster to call this for all the scanners first, and then
 * a second loop call the \c jsEndSendParameterFileToScanner() function.
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsGetErrorMessage()
 * \see \c #jsEndSendParameterFileToScanner()
 * \see \c #jsGetNumberOfErrorMessages()
 * \see \c #jsGetParameterFileFromScanner()
 *
 * \param jc The connection to the scanner.
 * \param filename The name of the file to send to the scanner.
 *
 * \return 0 on success.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c filename is \c NULL.
 */
JCAM_DLL_API int STDCALL jsBeginSendParameterFileToScanner(JCONNECTION const jc, char const * const filename);
/**
 * \ingroup parameters
 * \brief Completes the sending of parameters.  Should always and only be called after a call to \c jsBeginSendParameterFileToScanner() 
 * or \c jsBeginSendParametersToScanner().
 *
 * If \c #OPERATION_FAILURE is returned, the error messages should be checked
 * using \c jsGetNumberOfErrorMessages() and \c jsGetErrorMessage().  The error
 * messages are stored inside the opaque \c #JCONNECTION.
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsGetErrorMessage()
 * \see \c #jsBeginSendParameterFileToScanner()
 * \see \c #jsBeginSendParametersToScanner()
 * \see \c #jsGetNumberOfErrorMessages()
 * \see \c #jsGetParameterFileFromScanner()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c filename is \c NULL.
 * \return \c #OPERATION_FAILURE on all other failures, including parse and file I/O errors.
 */
JCAM_DLL_API int STDCALL jsEndSendParametersToScanner(JCONNECTION const jc);

/**
 * \ingroup parameters
 * \brief Sends a parameter file to the scanhead.
 *
 * If \c #OPERATION_FAILURE is returned, the error messages should be checked
 * using \c jsGetNumberOfErrorMessages() and \c jsGetErrorMessage().  The error
 * messages are stored inside the opaque \c #JCONNECTION.
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsGetErrorMessage()
 * \see \c #jsSendParametersToScanner()
 * \see \c #jsGetNumberOfErrorMessages()
 * \see \c #jsGetParameterFileFromScanner()
 *
 * \param jc The connection to the scanner.
 * \param filename The name of the file to send to the scanner.
 *
 * \return 0 on success.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c filename is \c NULL.
 * \return \c #OPERATION_FAILURE on all other failures, including parse and file I/O errors.
 */
JCAM_DLL_API int STDCALL jsSendParameterFileToScanner(JCONNECTION const jc, char const * const filename);
/**
 * \ingroup parameters
 * \brief Sends parameters to the scanhead.
 *
 * If \c #OPERATION_FAILURE is returned, the error messages should be checked
 * using \c jsGetNumberOfErrorMessages() and \c jsGetErrorMessage().  The error
 * messages are stored inside the opaque \c #JCONNECTION.
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsGetErrorMessage()
 * \see \c #jsGetNumberOfErrorMessages() 
 * \see \c #jsSendParameterFileToScanner()
 * \see \c #jsGetParameterFileFromScanner()
 *
 * \param jc The connection to the scanner.
 * \param parameters A NULL-terminated string with the parameters to send to the scanner.
 *
 * \return 0 on success.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c parameters is \c NULL.
 * \return \c #OPERATION_FAILURE on all other failures, including parse and file I/O errors.
 */
JCAM_DLL_API int STDCALL jsSendParametersToScanner(JCONNECTION const jc, char const * const parameters);
/**
 * \ingroup parameters
 * \brief Starts sending a parameter string to the scanhead.  Must be followed with a called to \c jsEndSendParametersToScanner().
 * 
 * When connecting to large group of scanners, it is must faster to call this for all the scanners first, and then
 * a second loop call the \c jsEndSendParametersToScanner() function.
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsGetErrorMessage()
 * \see \c #jsEndSendParametersToScanner()
 * \see \c #jsGetNumberOfErrorMessages()
 * \see \c #jsGetParameterFileFromScanner()
 *
 * \param jc The connection to the scanner.
 * \param filename The name of the file to send to the scanner.
 *
 * \return 0 on success.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c filename is \c NULL.
 */
JCAM_DLL_API int STDCALL jsBeginSendParametersToScanner(JCONNECTION const jc, char const * const parameters);/**
 * \ingroup parameters
 * \brief Reads the current parameter file from the scanhead and writes them to the specified file.
 * May \b not be called while the scanner is in Synchronized Scanning Mode.<br>
 *
 * \see \c #jsSendParametersToScanner()
 * \see \c #jsSendParameterFileToScanner()
 *
 * \param jc the connection to the scanner.
 * \param filename The name of the file to save the parameters in.  The file will be created if it doesn't exist.
 *
 * \return 0 on success.
 * \return \c #SCANNER_FAILURE on all connection failures.
 * \return \c #INVALID_PARAMETER if \c jc or \c filename is \c NULL.
 * \return \c #OPERATION_FAILURE on all other failures.
 */
JCAM_DLL_API int STDCALL jsGetParameterFileFromScanner(JCONNECTION const jc, char const * const filename);
/**
 * \ingroup parameters
 * \brief Returns the number of error messages after a
 * \c jsSendParameterFileToScanner() call returned \c #OPERATION_FAILURE.
 *
 * \see \c #jsGetErrorMessage()
 * \see \c #jsSendParametersToScanner()
 * \see \c #jsSendParameterFileToScanner()
 *
 * \param jc The connection to the scanner.
 *
 * \return The number of error messages.
 * \return 0 if \c jc is \c NULL.
 */
JCAM_DLL_API size_t STDCALL jsGetNumberOfErrorMessages(JCONNECTION const jc);
/**
 * \ingroup parameters
 * \brief Returns the specified error message.
 *
 * Do not modify the returned string.
 *
 * \see \c #jsSendParametersToScanner()
 * \see \c #jsGetNumberOfErrorMessages()
 * \see \c #jsSendParameterFileToScanner()
 *
 * \param jc The connection to the scanner.
 * \param i The index of the error message to retrieve.
 *
 * \return A C string of the error message.
 * \return \c NULL if \c i is out of range or \c jc is \c NULL.
 */
JCAM_DLL_API char const * const STDCALL jsGetErrorMessage(JCONNECTION const jc, size_t i);
/**
 * \ingroup scannerConfiguration
 * \brief Reads the specified number of old position calibrations for the selected laser from the scanner.
 *
 * \see \c #jsSendPositionCalibrationN()
 * \see \c #jsReadPositionCalibrationN()
 *
 * \param jc The connection to the scanner.
 * \param laserIndex Which laser to read the position calibration for.
 * \param oldCalibrations An array of \c #jsOldCalibrationValue objects to store the old calibrations in.
 * \param nCalibrations The number of elements in jsOldCalibrationValue.
 *
 * \return The number of calibrations read on success.
 * \return 0 if there are no old calibrations.
 * \return \c #INVALID_PARAMETER if \c jc or \c oldCalibrations \c NULL or if \c laserIndex is invalid.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsReadOldPositionCalibrationsN(JCONNECTION const jc, jsLaserIndex const laserIndex, jsOldCalibrationValue oldCalibrations[], UINT32 nCalibrations);

/**
 * \ingroup scannerConfiguration
 * \brief Override the Laser Autoexposure parameters set in param.dat.  Values will be lost when a new param.dat file is loaded, or when the head restarts.
 *        Must be called for each scanner that you wish the values to apply to.
 *
 * \param jc The connection to the scanner.
 * \param minLaserOn The minimum length of time the laser will turn on.  Units milliseconds. Range 0.01 - 650.0. Default 0.01.
 * \param maxLaserOn The maximum length of time the laser will turn on. Units milliseconds. Range 0.01 - 650.0. Default 5.0.
 * \param defaultLaserOn When too few points are found, the laser on time will revert to this default value. 
 *		                 Units milliseconds. Range 0.01 - 650.0. Default average of min and max
 * \param saturatedIntensity This is how bright a data point must be to be considered saturated. 
 *                           If over saturatedPercentage of the data points are allowed to have a brightness over this level. 
 *                           Range 0 - 1023. Default 800.
 * \param saturatedPercentaged The percentage of found points that the scanner will attempt to keep over saturatedIntensity. Range 1 - 100. Default 30.
 * \param laserThreshold A point must be at least laserThreshold brightness to be detect a laser point. Range 0 - 1023. Default 120.
 * \param autoExposeLaser Enable laser autoexposure if true.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc  \c NULL or if any of the settings parameters are invalid.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsOverrideLaserAutoexposure(JCONNECTION const jc, double minLaserOn, double maxLaserOn, double defaultLaserOn, 
													 int staturatedIntensity, int saturatedPercentage, int laserThreshold, bool autoexposeLaser);
/**
 * \ingroup scannerConfiguration
 * \brief Override the Scan Window parameters set in param.dat.  Values will be lost when a new param.dat file is loaded, or when the head restarts.
 *        Must be called for each scanner that you wish the values to apply to.
 *
 * \param jc The connection to the scanner.
 * \param windowTop Points with Y values greater than this will be ignored 
 * \param windowBottom Points with Y values less than this will be ignored 
 * \param windowLeft Points with X values less than this will be ignored 
 * \param windowRight Points with X values greater than this will be ignored 
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc  \c NULL or if any of the settings parameters are invalid.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsOverrideScanWindow(JCONNECTION const jc, double windowTop, double windowBottom, double windowLeft, double windowRight); 
/**
 * \ingroup scannerConfiguration
 * \brief Override the Synchronized Scanning parameters set in param.dat.  Values will be lost when a new param.dat file is loaded, or when the head restarts.
 *        Must be called for each scanner that you wish the values to apply to.
 *
 * \param jc The connection to the scanner.
 * \param timeScannedInterval The time interval in seconds between scans in TimeSyncMode.  Range 0.0 to 167.7. Default 0.01.
 * \param encoderPulseInterval Distance between encoder puleses in EncoderSyncMode (Full quadrature, typically 4x the specified pulses per revolution).
 *                    default 1.0.
 * \param encoderScanInterval Distance between scans in EncoderSyncMode.  Must be greater than the encoderPulseInterval (ideally 100X or more)
 *                    default 1.0.
 * \param numberOfPhases If timeStaggeredScanning is false then delay between 0.0 and 1.0 of the entire encoderScanInterval.  Default 2. 
 * \param timeStaggeredScanning If true this scanner will stagger based on a fixed time delay.  Default false.
 * \param laser0Phase If timeStaggered scanning is true, then this laser will delay timeStaggerDelay ms after the encoder, or PulseMaster trigger.
 *                    otherwise the scanner will fire in the laser0Phase encoder phase.
 * \param laser1Phase If timeStaggered scanning is true, then this laser will delay timeStaggerDelay ms after the encoder, or PulseMaster trigger.
 *                    otherwise the scanner will fire in the laser0Phase encoder phase.
 * \param laser2Phase If timeStaggered scanning is true, then this laser will delay timeStaggerDelay ms after the encoder, or PulseMaster trigger.
 *                    otherwise the scanner will fire in the laser0Phase encoder phase.
 * \param laser3Phase If timeStaggered scanning is true, then this laser will delay timeStaggerDelay ms after the encoder, or PulseMaster trigger.
 *                    otherwise the scanner will fire in the laser0Phase encoder phase.
 * \param laser4Phase If timeStaggered scanning is true, then this laser will delay timeStaggerDelay ms after the encoder, or PulseMaster trigger.
 *                    otherwise the scanner will fire in the laser0Phase encoder phase.
 * \param untriggeredSyncScanning If true then the scanner won't wait for a start scan signal trigger, and instead start scanning immediately upon receiving the 
 *                                \c #jsEnterEncoderSyncMode() or \c jsEnterTimeSyncMode(). Default false.
 *							      This is not a reliable technique for multiple scanner systems if they enter scanning while the encoder is moving, because there may be 
 *                                a substantial difference between when various scanners receive the enter sync mode command. 
 * \param startScanRisingEdgeTriggered If true scanning will start on the rising edge of the start scan signal, otherwise on the falling edge. Default false.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc  \c NULL or if any of the settings parameters are invalid.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsOverrideSyncSettings(JCONNECTION const jc, double timeScannedInterval, double encoderPulseInterval, 
												double encoderScanInterval, int numberOfPhases, bool timeStaggeredScanning, 
												double laser0Phase, double laser1Phase, double laser2Phase, double laser3Phase, double laser4Phase, 
												bool untriggeredSyncScanning, bool startScanRisingEdgeTriggered); 

/**
 * \ingroup scannerConfiguration
 * \brief Reads the current scan window for the selected laser from the scanner.
 *
 * \since Revision 2138 / Scanner Build 1800
 *
 * \param jc The connection to the scanner.
 * \param laserIndex Which laser to read the position calibration for.
 * \param top Location to store window top in 1/1000 of inch or mm units (based on Units setting in paramters)
 * \param bottom Location to store window bottom in 1/1000 of inch or mm units
 * \param left Location to store window left in 1/1000 of inch or mm units
 * \param right Location to store window right in 1/1000 of inch or mm units
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc, \c top, \c bottom, \c left, or \c right is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */

JCAM_DLL_API int STDCALL jsReadScanWindowN(JCONNECTION const jc, jsLaserIndex const laserIndex, int *top, int *bottom, int *left, int *right);
/**
 * \ingroup scannerConfiguration
 * \brief Reads the current position calibration for the selected laser from the scanner.
 *
 * \see \c #jsSendPositionCalibrationN()
 * \see \c #jsReadOldPositionCalibrationsN()
 *
 * \since Revision 1932
 *
 * \param jc The connection to the scanner.
 * \param laserIndex Which laser to read the position calibration for.
 * \param xOffset Stores the value of the scanner's x offset.
 * \param yOffset Stores the value of the scanner's y offset.
 * \param roll Stores the value of the scanner's rotation.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc, \c xOffset, \c yOffset, or \c roll is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsReadPositionCalibrationN(JCONNECTION const jc, jsLaserIndex const laserIndex, double *xOffset, double *yOffset, double *roll);
/**
 * \ingroup scannerConfiguration
 * \brief Sends a position calibration for the selected laser to the scanner.
 *
 * \see \c #jsReadPositionCalibrationN()
 * \see \c #jsReadOldPositionCalibrationsN()
 *
 * \param jc The connection to the scanner.
 * \param laserIndex Which laser to send the position calibration for.
 * \param xOffset The new value of the scanner's x offset.
 * \param yOffset The new value of the scanner's y offset.
 * \param roll The new value of the scanner's rotation.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE on all connection failures.
 */
JCAM_DLL_API int STDCALL jsSendPositionCalibrationN(JCONNECTION const jc, jsLaserIndex const laserIndex, double const xOffset, double const yOffset, double const roll);
/**
 * \ingroup statusFunctions
 * \brief Reads status from the scanner into a data structure inside the opaque \c #JCONNECTION.
 *
 * \see \c #jsGetScannerStatusValue()
 * \see \c #jsGetStatusDescriptionFromScanner()
 *
 * \param jc The connection to the scanner.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 * \return \c #SCANNER_FAILURE if the connection closes.
 */
JCAM_DLL_API int STDCALL jsGetScannerStatusFromScanner(JCONNECTION const jc);
/**
 * \ingroup statusFunctions
 * \brief Gets the specified status value from the data structure inside the \c #JCONNECTION.
 *
 * \see \c #jsGetScannerStatusFromScanner()
 * \see \c #jsGetStatusDescriptionFromScanner()
 *
 * \param jc The connection to the scanner.
 * \param i The index of the status value to retrieve.
 * \param value Pointer to an int that will hold the value.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c value is \c NULL, or if \c i is out of range.
 */
JCAM_DLL_API int STDCALL jsGetScannerStatusValue(JCONNECTION const jc, size_t i, int *value);
/**
 * \ingroup statusFunctions
 * \brief Reads a descriptive text string from the scanner for the particular status index.
 *
 * The description length should be a minimum of 128 characters.
 * After this call, \c description will be properly null-terminated.
 * If \c description isn't long enough, the result will be truncated, but still properly null-terminated.
 * The first valid index is 0, the last valid index is 12.
 *
 * Invalid indexes will return the string "ERROR unsuported status type".
 *
 * \see \c #jsGetScannerStatusValue()
 * \see \c #jsGetScannerStatusFromScanner()
 *
 * \param jc The connection to the scanner.
 * \param i The index of the status name to retrieve.
 * \param description The character buffer that will contain the status name.
 * \param description_length The number of characters that \c description points to.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c jc or \c description is \c NULL, or if \c i is out of range.
 * \return \c #SCANNER_FAILURE if the connection closes.
 */
JCAM_DLL_API int STDCALL jsGetStatusDescriptionFromScanner(JCONNECTION const jc, UINT32 i, char * const description, UINT32 description_length);
/**
 * \ingroup scanAndImage
 * \brief Returns a pixel value from an image.
 *
 * This function checks to ensure that \c image isn't \c NULL, but doesn't do
 * any error checking to ensure that \c x and \c y are within bounds.<BR>
 * <code>(0, 0)</code> is the upper-left pixel.
 * <code>(image->numberHorizontal - 1, image->numberVertical - 1)</code> is the lower-right pixel.
 *
 * \param image The image to get a pixel value from.
 * \param x The x coordinate of the pixel to get.
 * \param y The y coordinate of the pixel to get.
 *
 * \return The greyscale pixel value in the range of 0 to 255.
 * \return 0 if \c image is \c NULL.
 */
JCAM_DLL_API int STDCALL jsGetImagePixel(jsImage const * const image, size_t const x, size_t const y);
/**
 * \ingroup scannerDiscovery
 * \brief Attempts to discover all scanners on the network.
 *
 * Up to \c nResponses from the scanners will be stored in \c responses.<BR>
 * Blocks for approximately one second while waiting for responses from the scanners.
 *
 * \param responses An array for holding responses from the scanners on the network.
 * \param nResponses The number of elements in \c responses.
 *
 * \return The number of responses on success.
 * \return -1 on all failures.
 */
JCAM_DLL_API int STDCALL jsFindAllScanners(jsResponsePacket responses[], const size_t nResponses);

#ifdef WIN32
JCAM_DLL_API int STDCALL jsFindAllScannersVB(LPSAFEARRAY lpsaResponses);
#endif

/**
 * \ingroup scannerDiscovery
 * \brief Attempts to discover the scanner(s) on the network with the specified Cable ID.
 *
 * If there is more than one scanner with the same Cable ID, they will both respond.<BR>
 * Up to \c nResponses from the scanners will be stored in \c responses.<BR>
 * Blocks for approximately one second while waiting for responses from the scanners.
 *
 * \param cableId The cable ID of the scanner(s) you want to discover.
 * \param responses An array for holding responses from the scanners on the network.
 * \param nResponses The number of elements in \c responses.
 *
 * \return The number of responses on success.
 * \return -1 on all failures.
 */
JCAM_DLL_API int STDCALL jsFindScannerByCableId(int const cableId, jsResponsePacket responses[], const size_t nResponses);
/**
 * \ingroup scannerDiscovery
 * \brief Attempts to discover the scanner on the network with the specified serial number.
 *
 * Since serial numbers are unique, there should only ever be one scanner responding to this search.<BR>
 * Up to \c nResponses from the scanners will be stored in \c responses.<BR>
 * Blocks for approximately one second while waiting for responses from the scanners.
 *
 * \param serialNumber The serial number of the scanner you want to discover.
 * \param responses An array for holding responses from the scanners on the network.
 * \param nResponses The number of elements in \c responses.
 *
 * \return The number of responses on success.
 * \return -1 on all failures.
 */
JCAM_DLL_API int STDCALL jsFindScannerBySerialNumber(int const serialNumber, jsResponsePacket responses[], const size_t nResponses);
/**
 * \ingroup scannerConfiguration
 * \brief Sets a scanner's static IP address.
 *
 * The IP address will be static; it will not depend on the Cable ID of the scanner.
 * If the scanner is moved to a different location with a different Cable ID, it will have the same IP address.
 *
 * \param serialNumber The serial number of the scanner whose IP address you want to set.
 * \param staticIpAddress <b>Must be in network byte order.</b> An integer representing the IP address the scanner will use.
 * \param netmask <b>Must be in network byte order.</b> An integer representing the IP netmask the scanner will use.
 *
 * \return 0 on success.
 * \return -1 on all failures.
 */
JCAM_DLL_API int STDCALL jsSetStaticIpInt(UINT32 const serialNumber, UINT32 const staticIpAddress, UINT32 const netmask);
/**
 * \ingroup scannerConfiguration
 * \brief Sets a scanner's static IP address.
 *
 * The IP address will be static; it will not depend on the Cable ID of the scanner.
 * If the scanner is moved to a different location with a different Cable ID, it will have the same IP address.
 *
 * \param serialNumber The serial number of the scanner whose IP address you want to set.
 * \param staticIpAddress The IP address the scanner will use.
 * \param netmask The IP netmask the scanner will use.
 *
 * \return 0 on success.
 * \return \c INVALID_PARAMETER if \c baseIpAddress or \c netmask is \c NULL or invalid.
 * \return -1 on all other failures.
 */
JCAM_DLL_API int STDCALL jsSetStaticIpChar(UINT32 const serialNumber, char const * const staticIpAddress, char const * const netmask);
/**
 * \ingroup scannerConfiguration
 * \brief Sets a scanner's base IP address.
 *
 * The IP address will be determined by adding the scanner's Cable ID to the base IP address.
 *
 * \param serialNumber The serial number of the scanner whose IP address you want to set.
 * \param baseIpAddress <b>Must be in network byte order.</b> An integer representing the base IP address the scanner will use to determine its final IP address.
 * \param netmask <b>Must be in network byte order.</b> An integer representing the IP netmask the scanner will use.
 *
 * \return 0 on success.
 * \return -1 on all failures.
 */
JCAM_DLL_API int STDCALL jsSetBaseIpInt(UINT32 const serialNumber, UINT32 const baseIpAddress, UINT32 const netmask);
/**
 * \ingroup scannerConfiguration
 * \brief Sets a scanner's base IP address.
 *
 * The IP address will be determined by adding the scanner's Cable ID to the base IP address.
 *
 * \param serialNumber The serial number of the scanner whose IP address you want to set.
 * \param baseIpAddress The base IP address the scanner will use to determine its final IP address.
 * \param netmask The IP netmask the scanner will use.
 *
 * \return 0 on success.
 * \return \c INVALID_PARAMETER if \c baseIpAddress or \c netmask is \c NULL or invalid.
 * \return -1 on all other failures.
 */
JCAM_DLL_API int STDCALL jsSetBaseIpChar(UINT32 const serialNumber, char const * const baseIpAddress, char const * const netmask);
/**
 * \ingroup scannerConfiguration
 * \brief Overrides the cable ID physically wired into a scanner.
 *
 * The new cable ID is persistent.
 * If the scanner is rebooted, it will ignore the physically wired in cable ID.
 * If the scanner is set to use Base+IP addressing, the scanner's IP address will change according to the new IP address.
 *
 * \see \c #jsClearCableId()
 *
 * \param serialNumber The serial number of the scanner to change.
 * \param cableId The cable ID you wish the scanner to have.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER if \c cableId is over 31.
 * \return -1 on all other failures.
 */
JCAM_DLL_API int STDCALL jsSetCableId(UINT32 const serialNumber, UINT32 const cableId);
/**
 * \ingroup scannerConfiguration
 * \brief Reverts to the cable ID physically wired into a scanner.
 *
 * If the scanner is set to use Base+IP addressing, the scanner's IP address will change according to the new IP address.
 *
 * \see \c #jsSetCableId()
 *
 * \param serialNumber The serial number of the scanner to change.
 *
 * \return 0 on success.
 * \return \c #INVALID_PARAMETER \c cableId is over 31.
 * \return -1 on all other failures.
 */
JCAM_DLL_API int STDCALL jsClearCableId(UINT32 const serialNumber);

/**
 * \ingroup synchronizedScanning
 * \brief Returns if a scanner is in Synchronized Mode.
 *
 * \param jc The connection to the scanner.
 *
 * \return 1 if scanner is in Synchronized Mode.
 * \return 0 if scanner is not in Synchronized Mode.
 * \return \c #INVALID_PARAMETER if \c jc is \c NULL.
 */
JCAM_DLL_API int STDCALL jsIsInSyncMode(JCONNECTION const jc);

#ifdef __cplusplus
	} //namespace joescan {
#endif //__cplusplus

#if (defined JCAM_STATIC_LIB && defined WIN32)
#	include <windef.h>

	/**
	 * \ingroup miscellaneousFunctions
	 * \brief Initializes TCP/IP communications.
	 * 
	 * In Windows, TCP/IP communications must be initialized.
	 * No one knows why, it simply is that way.
	 * 
	 * If you are using this as a statically linked library, call
	 * this function before you use the functions in this library,
	 * otherwise bad things will happen.
	 * 
	 * If you are using this as a dynamically linked library (DLL),
	 * you don't need to call this function, the library loader will
	 * do it for you.
	 *
	 * \since 1.24
	 * 
	 * \return \c TRUE on success.
	 * \return \c FALSE on failures.
	 */
	BOOL jsInitialize();


	/**
	 * \ingroup miscellaneousFunctions
	 * \brief Gracefully ends TCP/IP communications.
	 * 
	 * After a succesful call to \c jsInitialize(), this function can
	 * be called if no futher calls to joescan API functions will
	 * occur.
	 * 
	 * 
	 * \since 1.24
	 */
	void jsCleanup();
#endif // (defined JCAM_STATIC_LIB && defined WIN32)

#ifdef __cplusplus
	} //extern "C" {
#endif //__cplusplus

#endif //JCAM_DLL
